<?php
/*
    Copyright 2007, 2008 University of Strathclyde
        
    This file is part of the SIMPLE Platform.

    The SIMPLE Platform is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The SIMPLE Platform is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the SIMPLE Platform.  If not, see <http://www.gnu.org/licenses/>.
    
*/    

/**
 * @copyright 2006 The Law School, University of Strathclyde
 * @author Michael Hughes
 * @package SIMPLE
 * @subpackage Objects
 */
 
 /**
  * Manages connection to underlying database using MDB2 Database abstraction library.
  */
  ini_set('mssql.textlimit', 2147483647);
  ini_set('mssql.textsize', 2147483647);
	class Database{
		/**
		*@var MDB2 An MDB2 database abstraction object.
		*/
		var $database = null;
		/**
		 * Constructor
		 * @global config 
		 Stores configuration information about the installation.
		 */
		function __construct() {
			global $config;
			$dsn = array(
				'phptype'=>$config[DATABASE_TYPE],
				'username'=>$config[DATABASE_USERNAME],
				'password'=>$config[DATABASE_PASSWORD],
				'port'=>$config[DATABASE_PORT],
				'hostspec'=>$config[DATABASE_HOST],
				'database'=>$config[DATABASE_NAME]
			);
			//echo join(",",$dsn) .'<br/>';
			
			$this->database = &MDB2::factory($dsn);
			//echo $this->database;
			//$this->database->setOption('debug',true);
			if (PEAR::isError($this->database)) {
				trace ("Database Constructor: ".$this->database->getMessage());
			}
		}
		/**
		 * Disconnects nicely from the database when the object is destructed.
		 */
		
		function __destruct() {
			$this->disconnect();
		}
		function dump($val) {
    echo '<pre>';
    print_r($val);
    echo '</pre>';
    }
		/**
		 * Performs a query against the database.
		 * @param string $query SQL query to perform against database.
		 * @return MDB2_Result object containing the result of the operation.
		 */
		function query($query) {
		  if (is_null($this->database) or PEAR::isError($this->database)) {
			 echo $this->database->getMessage();
			 print_r($this->database);
				die('Database object is null');
			}
			$result =& $this->database->query($query);
			if (PEAR::isError($result))
			{
				trace('<span style="background-color:red">Database Query: '.$result->getMessage().'</span>');
				print ($result->getMessage());
//				dump(debug_backtrace());
			}
			return $result;
		}
		/**
		 * Performs a query against the database and returns the results as an associative array.
		 * @param string $query SQL query to perform against database.
		 * @return array Associative array containing all the rows returned by the query.
		 */
		function queryAssoc($query,$start=null,$limit=null) {		
			if (is_null($this->database) or PEAR::isError($this->database)) {
			 echo $this->database->getMessage();
			 //print_r($this->database);
				die('Database object is null');
			}
			//print_r($this->database);
			//echo $query;
			if (!is_null($limit)){
			 $this->database->setLimit($limit,$start);
			}
			$result =& $this->database->query($query);
			
			$array = array();
			if(PEAR::isError($result)) {
				echo('<p style="background-color:red">Database Query: '.$result->getMessage().$query.'</p>');
				//echo '<p>'.print_r(debug_backtrace()).'</p>';
				echo '<p>'.print_r($result).'</p>';
			}else {
				$array = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
				$result->free();
			}
			return $array;
		}
		/**
		* @return bool True if the query is executed or the error if it wasn't
		*/
		function execute($query) {
		  if (is_null($this->database) or PEAR::isError($this->database)) {
			 echo $this->database->getMessage();
			 print_r($this->database);
				die('Database object is null');
			}
			$result =& $this->database->exec($query);
			if (PEAR::isError($result)) {
				trace('<span style="background-color:red">Database Execute: '.$result->getMessage().' '.$query);
        //dump(debug_backtrace());		
        echo "<P>$query</p>";
        echo  "<P>".$result->getMessage()."</p>";
        die('Problem Executing statement');		
			}
			return true;
		}	
		/*
		* quotes the a string for safe insertion into database
		* @param string 
		* @return string escaped string
		*/
		
		function quote($string) {
			$string = $this->database->escape($string);
			if (PEAR::isError($string)) {
		    	trace("Database escape(): ".$string->getMessage());
			}	
			return $string;
		}
				
		/** 
		 * Disconnects from the database.
		 */
		function disconnect(){
/*		  if (PEAR::isError($this->database)) {
        die($this->database->getMessage());
      }
      */
			if (is_null($this->database)) {
			 echo $this->database->getMessage();
			 print_r($this->database);
				die('Database object is null');
			}
			if(!is_null($this->database)) {
				$this->database->disconnect();
			}
		}
		/*
		/**
		 * Quotes the input parameter
		 *
		function Quote($in){
			return $this->database->quote($in);
		}
		*/
	}
	/**
	* @global Database $database
	*/
	$database = new Database();
	//echo $database->database;
?>
